# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .enum_invoice_sat_invoice_type import EnumInvoiceSatInvoiceType
from .enum_invoice_sat_payment_method import EnumInvoiceSatPaymentMethod
from .enum_invoice_type import EnumInvoiceType
from .invoice_detail_sat import InvoiceDetailSat
from .invoice_warnings_sat import InvoiceWarningsSat
from .invoices_payments_sat import InvoicesPaymentsSat
from .invoices_payroll_sat import InvoicesPayrollSat


class InvoiceWithIdSat(pydantic.BaseModel):
    id: typing.Optional[str] = pydantic.Field(
        description=("Belvo's unique identifier used to reference the current invoice.\n")
    )
    link: typing.Optional[str] = pydantic.Field(description=("The `link.id` the invoice belongs to.\n"))
    collected_at: typing.Optional[str] = pydantic.Field(
        description=("The ISO-8601 timestamp when the data point was collected.\n")
    )
    created_at: typing.Optional[str] = pydantic.Field(
        description=("The ISO-8601 timestamp of when the data point was last updated in Belvo's database.\n")
    )
    invoice_identification: typing.Optional[str] = pydantic.Field(
        description=("The fiscal institution's unique ID for the invoice.\n")
    )
    invoice_date: typing.Optional[str] = pydantic.Field(description=("The date of the invoice.\n"))
    status: typing.Optional[str] = pydantic.Field(
        description=("The status of the invoice. Can be either *Vigente* (valid) or *Cancelado* (cancelled).\n")
    )
    invoice_type: EnumInvoiceSatInvoiceType
    type: EnumInvoiceType
    sender_id: typing.Optional[str] = pydantic.Field(description=("The fiscal ID of the invoice sender\n"))
    sender_name: typing.Optional[str] = pydantic.Field(description=("The name of the invoice sender.\n"))
    sender_tax_fraud_status: typing.Optional[str] = pydantic.Field(
        description=(
            "Indicates whether or not the sender is on SAT's tax fraud list for having submitted incorrect data, having outstanding payments, or having conducted business that is in violation of the fiscal institution's regulations.<br><br>\n"
            "SAT updates the tax fraud list every three months. <br><br>\n"
            "For more information regarding the reason's a taxpayer can be put on the tax fraud list, please see [Article 69](http://omawww.sat.gob.mx/cifras_sat/Paginas/datos/vinculo.html?page=ListCompleta69.html) and [Article 69-B](http://omawww.sat.gob.mx/cifras_sat/Paginas/datos/vinculo.html?page=ListCompleta69B.html) of Mexico's Código Fiscal de la Federación. <br><br>\n"
            "Possible statuses are:\n"
            "- `INVESTIGATING` <br> The fiscal institution has identified irregularities and open an investigation regarding the taxpayer. <br> - `DISMISSED` <br> The fiscal institution has investigated the taxpayer and declared them innocent. <br> - `CONFIRMED` <br> The fiscal institution has confirmed that the taxpayer is guilty. <br> - `OVERTURNED` <br> The fiscal institution has reassessed a previously confirmed taxpayer and, based on new evidence, has taken the taxpayer off the tax fraud list. <br> - `NO_TAX_FRAUD_STATUS` <br> The receiver or sender is not found in the list (in other words, they are complying with the fiscal institution's regulations).\n"
        )
    )
    receiver_id: typing.Optional[str] = pydantic.Field(description=("The fiscal ID of the invoice receiver.\n"))
    receiver_name: typing.Optional[str] = pydantic.Field(description=("The name of the invoice receiver.\n"))
    receiver_tax_fraud_status: typing.Optional[str] = pydantic.Field(
        description=(
            "Indicates whether or not the receiver is on SAT's tax fraud list for having submitted incorrect data, having outstanding payments, or having conducted business that is in violation of the fiscal institution's regulations.<br><br>\n"
            "SAT updates the tax fraud list every three months. <br><br>\n"
            "For more information regarding the reason's a taxpayer can be put on the tax fraud list, please see [Article 69](http://omawww.sat.gob.mx/cifras_sat/Paginas/datos/vinculo.html?page=ListCompleta69.html) and [Article 69-B](http://omawww.sat.gob.mx/cifras_sat/Paginas/datos/vinculo.html?page=ListCompleta69B.html) of Mexico's Código Fiscal de la Federación. <br><br>\n"
            "Possible statuses are:\n"
            "- `INVESTIGATING` <br> The fiscal institution has identified irregularities and open an investigation regarding the taxpayer. <br> - `DISMISSED` <br> The fiscal institution has investigated the taxpayer and declared them innocent. <br> - `CONFIRMED` <br> The fiscal institution has confirmed that the taxpayer is guilty. <br> - `OVERTURNED` <br> The fiscal institution has reassessed a previously confirmed taxpayer and, based on new evidence, has taken the taxpayer off the tax fraud list. <br> - `NO_TAX_FRAUD_STATUS` <br> The receiver or sender is not found in the list (in other words, they are complying with the fiscal institution's regulations).\n"
        )
    )
    cancelation_status: typing.Optional[str] = pydantic.Field(
        description=("If the invoice is cancelled, this field indicates the status of the cancellation.\n")
    )
    cancelation_update_date: typing.Optional[str] = pydantic.Field(
        description=("The date of the invoice cancelation.\n")
    )
    certification_date: typing.Optional[str] = pydantic.Field(description=("The date of the fiscal certification.\n"))
    certification_authority: typing.Optional[str] = pydantic.Field(
        description=("The fiscal ID of the certification provider.\n")
    )
    payment_type: typing.Optional[str] = pydantic.Field(
        description=(
            "The payment type code used for this invoice, as defined by the country legal entity.\n"
            "\n"
            "- 🇲🇽 Mexico [SAT catalog reference article](https://developers.belvo.com/docs/sat-catalogs#payment-type)\n"
        )
    )
    payment_type_description: typing.Optional[str] = pydantic.Field(description=("*This field has been deprecated.*\n"))
    payment_method: typing.Optional[EnumInvoiceSatPaymentMethod]
    payment_method_description: typing.Optional[str] = pydantic.Field(
        description=(
            "*This field has been deprecated.*\n"
            "\n"
            "*The description of the payment method used for this invoice.*\n"
        )
    )
    usage: typing.Optional[str] = pydantic.Field(
        description=(
            "The invoice's usage code, as defined by the legal entity of the country. \n"
            "\n"
            "- 🇲🇽 Mexico [SAT catalog reference article](https://developers.belvo.com/docs/sat-catalogs#usage)\n"
        )
    )
    version: typing.Optional[str] = pydantic.Field(description=("The CFDI version of the invoice.\n"))
    place_of_issue: typing.Optional[str] = pydantic.Field(
        description=("The postcode of where the invoice was issued.\n")
    )
    invoice_details: typing.List[InvoiceDetailSat] = pydantic.Field(
        description=("A list of descriptions for each item (purchased product or service provided) in the invoice.\n")
    )
    currency: typing.Optional[str] = pydantic.Field(
        description=(
            "The currency of the invoice. For example:\n"
            " \n"
            " - 🇧🇷 BRL (Brazilian Real)\n"
            " - 🇨🇴 COP (Colombian Peso)\n"
            " - 🇲🇽 MXN (Mexican Peso)\n"
            " - 🇺🇸 USD (United States Dollar)\n"
        )
    )
    subtotal_amount: typing.Optional[float] = pydantic.Field(
        description=("The pretax amount of this invoice (sum of each item's `pre_tax_amount`).\n")
    )
    exchange_rate: typing.Optional[float] = pydantic.Field(
        description=("The exchange rate used in this invoice for the currency.\n")
    )
    tax_amount: typing.Optional[float] = pydantic.Field(
        description=("The amount of tax for this invoice (sum of each item's `tax_amount`).\n")
    )
    discount_amount: typing.Optional[float] = pydantic.Field(
        description=("The total amount discounted in this invoice.\n")
    )
    total_amount: typing.Optional[float] = pydantic.Field(
        description=("The total amount of the invoice (`subtotal_amount` + `tax_amount` - `discount_amount`)\n")
    )
    payments: typing.List[InvoicesPaymentsSat] = pydantic.Field(
        description=("A list detailing all the invoice payments.\n")
    )
    payroll: InvoicesPayrollSat
    folio: typing.Optional[str] = pydantic.Field(
        description=("The internal control number that the taxpayer assigns to the invoice.\n")
    )
    xml: typing.Optional[str] = pydantic.Field(description=("XML of the invoice document.\n"))
    warnings: typing.Optional[InvoiceWarningsSat]
    sender_blacklist_status: typing.Optional[str] = pydantic.Field(
        description=("This field has been deprecated. Please use `sender_tax_fraud_status` instead.\n")
    )
    receiver_blacklist_status: typing.Optional[str] = pydantic.Field(
        description=("This field has been deprecated. Please use `receiver_tax_fraud_status` instead.\n")
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}
