# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class TaxAssessmentIndividual(pydantic.BaseModel):
    """
    Object containing the calculated tax assessment of the tax payer. This includes the total taxable income, the income tax applied, and taxes already withheld.
    """

    fortuitous_profit_tax: float = pydantic.Field(
        description=("The tax applied on your unexpected income (such as lottery wins or house sales).\n")
    )
    total_tax_on_taxable_net_income: float = pydantic.Field(
        description=(
            "The calculated total tax that can be applied on the tax payer's taxable income (total income - exemptions - deductions).\n"
        )
    )
    net_income_tax: float = pydantic.Field(
        description=(
            "After additional deductions that you can apply, this will be the net income tax. If not further deduction are identified, this value will be the same as `total_tax_on_taxable_net_income`.\n"
        )
    )
    total_tax_due: float = pydantic.Field(
        description=(
            "After further deductions, this is the final calculated tax that the taxpayer is required to pay.\n"
        )
    )
    previous_year_balance: float = pydantic.Field(
        description=(
            "Only applicable for DIAN.\n"
            "\n"
            "\n"
            'The amount the tax payer has as a "credit" fromt he previous year (this is equal to the `balance_refundable`) of the previous year.\n'
        )
    )
    total_withheld_tax: float = pydantic.Field(
        description=("The total tax already withheld in the current fiscal year.\n")
    )
    balance_payable: float = pydantic.Field(description=("How much the tax payer is required to pay.\n"))
    balance_refundable: float = pydantic.Field(
        description=(
            "How much the tax payer is expected to receive. For DIAN, this will count as credit for the next fiscal year (see `previous_year_balance`).\n"
        )
    )
    total_payment: float = pydantic.Field(
        description=("The total the tax payer is required to pay, taking into account deductions and fiscal credits.\n")
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}
